<!--
* Copyright (c) 2017, 2021 IBM Corp. and others
*
* This program and the accompanying materials are made
* available under the terms of the Eclipse Public License 2.0
* which accompanies this distribution and is available at
* https://www.eclipse.org/legal/epl-2.0/ or the Apache
* License, Version 2.0 which accompanies this distribution and
* is available at https://www.apache.org/licenses/LICENSE-2.0.
*
* This Source Code may also be made available under the
* following Secondary Licenses when the conditions for such
* availability set forth in the Eclipse Public License, v. 2.0
* are satisfied: GNU General Public License, version 2 with
* the GNU Classpath Exception [1] and GNU General Public
* License, version 2 with the OpenJDK Assembly Exception [2].
*
* [1] https://www.gnu.org/software/classpath/license.html
* [2] https://openjdk.org/legal/assembly-exception.html
*
* SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH
* Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
-->

# Eclipse OpenJ9 version 0.11.0 release notes

These release notes support the [Eclipse OpenJ9 0.11.0 release plan](https://projects.eclipse.org/projects/technology.openj9/releases/0.11.0/plan).


## Binaries and supported environments

OpenJ9 release 0.11.0 supports OpenJDK 8 and OpenJDK 11. Binaries are available at the AdoptOpenJDK project:

- [OpenJDK with OpenJ9 version 8](https://adoptopenjdk.net/archive.html?variant=openjdk8&jvmVariant=openj9)
- [OpenJDK with OpenJ9 version 11](https://adoptopenjdk.net/archive.html?variant=openjdk11&jvmVariant=openj9)

OpenJ9 release 0.11.0 provides limited support for the MacOS platform on OpenJDK 11. Early builds of OpenJDK 11 with OpenJ9 on MacOS are available at
the AdoptOpenJDK project at the following link:  

- [OpenJDK v11 with OpenJ9](https://adoptopenjdk.net/nightly.html?variant=openjdk11&jvmVariant=openj9)

All builds are tested against the OpenJ9 functional verification (FV) test suite, the OpenJDK test suites, and additional tests at AdoptOpenJDK.

To learn more about support for OpenJ9 releases, including OpenJDK levels and platform support, see [Supported environments](https://eclipse.org/openj9/docs/openj9_support/index.html).


## Notable changes in this release

The following table covers notable changes in v0.11.0. Further information about these changes can be found in the [user documentation](https://www.eclipse.org/openj9/docs/version0.11/).

<table cellpadding="4" cellspacing="0" summary="" width="100%" rules="all" frame="border" border="1"><thead align="left">
<tr valign="bottom">
<th valign="bottom">Issue number</th>
<th valign="bottom">Description</th>
<th valign="bottom">Version / Platform</th>
<th valign="bottom">Impact</th>
</tr>
</thead>
<tbody>

<tr><td valign="top"><a href="https://github.com/eclipse-openj9/openj9/issues/3118">#3118</a></td>
<td valign="top">OpenSSL V1.1.1 support</td>
<td valign="top">OpenJDK8 (All platforms)</td>
<td valign="top">OpenSSL support is now available for improved cryptographic performance for the CBC, Digest, and GCM algorithms.  New command line options are available for disabling or enabling the algorithms.</td>
</tr>

<tr><td valign="top"><a href="https://github.com/eclipse-openj9/openj9/issues/2862">#2862</a></td>
<td valign="top">The default shared cache and cache snapshot directory is changed to the user's home directory</td>
<td valign="top">OpenJDK11 (AIX, Linux)</td>
<td valign="top">This change is for non-Windows platforms and applies only if `-Xshareclasses:groupAccess` is not used. If `groupAccess` is used, the directory that gets set remains as `/tmp/javasharedresources/`.</td>
</tr>

<tr><td valign="top"><a href="https://github.com/eclipse-openj9/openj9/issues/2862">#2862</a></td>
<td valign="top">Two new `-Xshareclasses` suboptions are introduced </td>
<td valign="top">OpenJDK8 and later (All platforms)</td>
<td valign="top">`-Xshareclasses:bootClassesOnly` prevents non-bootstrap class loaders caching classes in the shared classes cache. `-Xshareclasses:fatal` prevents the shared classes cache being started if an error occurs. </td>
</tr>

<tr><td valign="top"><a href="https://github.com/eclipse-openj9/openj9/issues/3184">#3184</a></td>
<td valign="top">VM container-awareness is now enabled by default</td>
<td valign="top">OpenJDK8 and later (Linux)</td>
<td valign="top">In earlier releases, this behavior was enabled by setting the `-XX:+UseContainerSupport` option. This setting is now the default. When the VM is running in a container, and a memory limit is set, the VM allocates more memory to the Java heap. </td>
</tr>

<tr><td valign="top"><a href="https://github.com/eclipse-openj9/openj9/issues/3054">#3054</a></td>
<td valign="top">Concurrent scavenge mode is now available on Linux x86</td>
<td valign="top">OpenJDK8 and later (x86-64: Linux only)</td>
<td valign="top">When this mode is enabled, (`-Xgc:concurrentScavenge`) the VM attempts to reduce GC pause-times for response-time sensitive, large heap applications. This mode is available only on compressed references builds. Attempting to use this option on a non-compressed references build results in an *unrecognized option* error message. </td>
</tr>

<tr><td valign="top"><a href="https://github.com/eclipse-openj9/openj9/issues/3062">#3062</a></td>
<td valign="top">New command line option `-XX:[+|-]PositiveIdentityHash`</td>
<td valign="top">OpenJDK8 and later (All platforms)</td>
<td valign="top">When enabled, identity hash codes (`System.identityHashCode` / `Object.hashCode`) always return non-negative values. Because the performance of identity hash-intensive operations might be impacted, this option is not enabled by default.</td>
</tr>

</table>


## Known Issues

The v0.11.0 release contains the following known issues and limitations:

<table cellpadding="4" cellspacing="0" summary="" width="100%" rules="all" frame="border" border="1">
<thead align="left">
<tr valign="bottom">
<th valign="bottom">Issue number</th>
<th valign="bottom">Description</th>
<th valign="bottom">Platform</th>
<th valign="bottom">Impact</th>
<th valign="bottom">Workaround</th>
</tr>
</thead>
<tbody>

<tr><td valign="top"><a href="https://github.com/eclipse-openj9/openj9/issues/378">#378</a></td>
<td valign="top">DDR support</td>
<td valign="top">AIX and macOS</td>
<td valign="top">Inability to diagnose problems with the VM, garbage collector, or JIT.</td>
<td valign="top">None</td>
</tr>

<tr><td valign="top"><a href="https://github.com/ibmruntimes/openj9-openjdk-jdk8/issues/129">#129</a></td>
<td valign="top">OpenSSL support</td>
<td valign="top">AIX</td>
<td valign="top">Bundling OpenSSL on AIX can result in problems like https://github.com/ibmruntimes/openj9-openjdk-jdk8/issues/129.</td>
<td valign="top">None</td>
</tr>

<tr><td valign="top"><a href="https://github.com/eclipse-openj9/openj9/issues/54">#54</a></td>
<td valign="top">Signal handling</td>
<td valign="top">All platforms</td>
<td valign="top">Currently, shutdown signals (<code>SIGINT</code>, <code>SIGHUP</code> and <code>SIGTERM</code>) and <code>SIGCONT</code> are fully supported on Unix platforms (pLinux, zLinux, xLinux, AIX, and z/OS). Support for other POSIX signals is pending. See <code>SunMiscSignalTest.java</code> for the list of signals that need to be supported.</td>
<td valign="top">None</td>
</tr>

<tr><td valign="top"><a href="https://github.com/eclipse-openj9/openj9/issues/2507">#2507</a></td>
<td valign="top">Restriction analyzing system (core) dumps</td>
<td valign="top">Linux and Windows</td>
<td valign="top">You must use a 32-bit JVM to look at a 32-bit core, and a 64-bit JVM to look at a 64-bit core. This restriction will be fixed in a later version of OpenJ9.</td>
<td valign="top">None</td>
</tr>

<tr><td valign="top"><a href="https://github.com/eclipse-openj9/openj9/issues/479">#479</a></td>
<td valign="top">Non-compressed references VM builds not available</td>
<td valign="top">AIX, Windows, Linux on Z, Linux on Power</td>
<td valign="top">Non-compressed references VM builds, which support heaps larger than 57GB, are built on Linux on x86. </td>
<td valign="top">Manual builds on other platforms are possible by following our <a href="https://github.com/eclipse-openj9/openj9/blob/master/buildenv/Build_Instructions_V8.md">detailed build instructions</a>.</td>
</tr>

<tr><td valign="top"><a href="https://github.com/eclipse-openj9/openj9/issues/897">#897</a></td>
<td valign="top">Javadoc not building some OpenJ9 extensions.</td>
<td valign="top">All platforms</td>
<td valign="top">Some API documentation is missing in the <code>build/&lt;platform&gt;/docs</code> directory.</td>
<td valign="top">None</td>
</tr>

<tr><td valign="top"><a href="https://github.com/eclipse-openj9/openj9/issues/898">#898</a></td>
<td valign="top">Javadoc building some unsupported OpenJ9 extensions.</td>
<td valign="top">All platforms</td>
<td valign="top">Some API documentation in <code>build/&lt;platform&gt;/docs</code> is not supported with OpenJ9.</td>
<td valign="top">None</td>
</tr>

<tr><td valign="top"><a href="https://github.com/ibmruntimes/openj9-openjdk-jdk8/issues/17">#17</a></td>
<td valign="top">OpenJDK with OpenJ9 builds before Java 11 contain extra shared libraries.</td>
<td valign="top">All platforms</td>
<td valign="top">Shared libraries must not be used in production.</td>
<td valign="top">None</td>
</tr>
</tbody>
</table>

## Other changes

A full commit history for this release is available at [Eclipse OpenJ9 v0.11.0](https://github.com/eclipse-openj9/openj9/releases/tag/openj9-0.11.0).
